<?php

namespace FM\Service;

use FM\Factory\Fm00004000TarefasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;

class Fm00004000TarefasService
{

    protected $TarefasFactory;
    protected $Environment;
    protected $newCode;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $connection;

    public function __construct(EntityManager $entityManager, Fm00004000TarefasFactory $TarefasFactory, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->TarefasFactory               = $TarefasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    // integração SM
    public function getContratos($parametros)
    {
        if (!empty($parametros['NRESTRUTURAM'])) {
            return $this->TarefasFactory->getContratos($parametros['NRESTRUTURAM']);
        }
    }

    public function getTarefasGrid()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->TarefasFactory->getTarefasGrid($params, $listaOperadores);
    }

    public function getClientes()
    {
        return $this->TarefasFactory->getClientes();
    }

    public function getTipoLocalTarefa($parametros)
    {
        $arrayClientes = (is_array($parametros['CDCLIENTE'])) ? $this->FindService->formatarArray($parametros['CDCLIENTE']) : "'". $parametros['CDCLIENTE']."'";
        return $this->TarefasFactory->getTipoLocalTarefa($arrayClientes);
    }

    public function getLocal($parametros)
    {
        $tipoLocal = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->TarefasFactory->getLocal($tipoLocal, "'".$parametros['CDCLIENTE']."'");
    }

    public function getChecklistTarefa($parametros)
    {
        $param = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->TarefasFactory->getChecklistTarefa($param);
    }

    public function getItensChecklist($parametros)
    {
        return $this->TarefasFactory->getItensChecklist($parametros);
    }

    public function getEscopo()
    {
        return $this->TarefasFactory->getEscopo();
    }

    public function getTarefasSelect($parametros)
    {
        $param = implode(',', $parametros['NRATIVIDADECHK']);
        return $this->TarefasFactory->getTarefasSelect($param);
    }


    public function getCriterios($parametros)
    {
        $param = array(
            "NRTAREFACHK"    => implode(',', $parametros['NRTAREFACHK']),
            "NRATIVIDADECHK" => implode(',', $parametros['NRATIVIDADECHK']),
            "NRCHECKLIST"    => $parametros['NRCHECKLIST']
        );
        return $this->TarefasFactory->getCriterios($param);
    }

    public function getFuncionario($parametros)
    {
        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $param = is_array($parametros['NRLOCALCLIENTE']) ? $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']) : "'".$parametros['NRLOCALCLIENTE']."'";
            return $this->TarefasFactory->getFuncionario($param);
        }
    }

    public function getSupervisorTarefa($parametros)
    {
        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $param = is_array($parametros['NRLOCALCLIENTE']) ? $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']) : "'".$parametros['NRLOCALCLIENTE']."'";
            return $this->TarefasFactory->getSupervisorTarefa($param);
        }
    }

    public function insertNovaTarefa($param)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG =  $this->Environment->getNrOrg();

            if ($param['OBSERVACOES'] == 1) {
                $observacoesTarefa = "N";
                $observacoesVistoria = "N";
            } elseif ($param['OBSERVACOES'] == 2) {
                $observacoesTarefa = "S";
                $observacoesVistoria = "N";
            } elseif ($param['OBSERVACOES'] == 3) {
                $observacoesTarefa = "N";
                $observacoesVistoria = "S";
            } else {
                $observacoesTarefa = "S";
                $observacoesVistoria = "S";
            }

            foreach ($param["DADOSTAREFAS"] as $row) {
                if ($row["GERATAREFA"] == 'S') {
                        $nrchecklistagend = $this->newCode->getCodeByTableName("FCM_CHECKLISTAGEND", $NRORG, 1, 10, true);
                        $dataInicial = new \DateTime($row['DATAINICIAL']);
                        $dataFinal = new \DateTime($row['DATAINICIAL']);
                        $params = array(
                            "NRCHECKLISTAGEND"      => $nrchecklistagend,
                            "NMCHECKLISTAGEND"      => $param['NOMETAREFA'],
                            "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                            "NRCHECKLIST"           => $param['NRCHECKLIST'],
                            "CDOPERADOR"            => $row['FUNCIONARIO'],
                            "IDSITUACAOAGEND"       => 1,
                            "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                            "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                            "CDOPERINCLUSAO"        => $this->Environment->getCdOperador(),
                            "IDOBRIGAOBSERVACAO"    => $observacoesTarefa,
                            "NRSEQCONTRATO"         => isset($row['NRSEQCONTRATO']) ? $row['NRSEQCONTRATO'] : null,
                            "IDTIPOMARCACAOTAREFA"  => $row['IDTIPOMARCACAOTAREFA'],
                            "IDTIPORECORRENCIA"     => $param['RECORRENCIA'],
                        );

                        $this->TarefasFactory->insertChecklist($params);

                        foreach ($param['NRTAREFACHK'] as $atividade) {
                            $nrtarefaagend = $this->newCode->getCodeByTableName("FCM_TAREFAAGEND", $NRORG, 1, 10, true);
                            $params = array(
                                "NRTAREFAAGEND" => $nrtarefaagend,
                                "NRCHECKLISTAGEND" => $nrchecklistagend,
                                "NRTAREFACHK" => $atividade,
                                "CDOPERADOR" => $row['FUNCIONARIO'],
                                "DTHORAAGEND" => $dataInicial->format('d/m/Y H:i'),
                                "DSOBSTAREFAAGEND" => null,
                                "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                                "NRTAREFARECORRENTE" =>  isset($param["NRTAREFARECORRENTE"]) ? $param["NRTAREFARECORRENTE"] : null,
                            );

                            $this->TarefasFactory->insertTarefa($params);

                            foreach ($param['NRCRITERIOTAREFACHK'] as $criterios) {
                                //valida se o critério realmente pertence à atividade que está sendo inserida acima.
                                $paramsValida = array(
                                    "NRTAREFACHK"      => $atividade,
                                    "NRCRITERIOTAREFA" => $criterios['NRCRITERIOTAREFA']
                                );
                                $valida = $this->TarefasFactory->validaCriterio($paramsValida);
                                if ($valida > 0) {
                                    $nrcriteriostaragend = $this->newCode->getCodeByTableName("FCM_CRITERIOTARAGEND", $NRORG, 1, 10, true);
                                    $params = array(
                                        "NRCRITERIOTARAGEND" => $nrcriteriostaragend,
                                        "NMCRITERIOTARAGEND" => $criterios['DSCRITERIOTAREFA'],
                                        "NRTAREFAAGEND" => $nrtarefaagend,
                                        "NRCRITERIOTAREFACHK" => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                                        "OPERADOR" => $this->Environment->getCdOperador()
                                    );

                                    $this->TarefasFactory->insertCriterio($params);
                                }
                            }
                        }
                        if ($param["VISTORIA"] == 1) {
                            $nrgerenciamentochk = $this->newCode->getCodeByTableName("FCM_GERENCIAMENTOCHK", $NRORG, 1, 10, true);
                            $params = array(
                                "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                                "NMGERENCIAMENTOCHK"    => $param['NOMETAREFA'],
                                "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                                "NRCHECKLIST"           => $param['NRCHECKLIST'],
                                "CDOPERADOR"            => $row['ENCARREGADO'],
                                "DTHORAAGEND"           => $dataInicial->format('d/m/Y H:i'),
                                "DTHRFINALAGEND"        => $dataFinal->format('d/m/Y H:i'),
                                "OPERADOR"              => $this->Environment->getCdOperador(),
                                "NRCHECKLISTAGEND"      => $nrchecklistagend,
                                "IDOBRIGAOBSERVACAO"    => $observacoesVistoria,
                                "NRSEQCONTRATO"         => (isset($row['NRSEQCONTRATO'])) ? $row['NRSEQCONTRATO'] : null,
                            );

                            $this->TarefasFactory->insertChk($params);

                            foreach ($param['NRTAREFACHK'] as $atividade) {
                                $nrtarefaagendchk = $this->newCode->getCodeByTableName("FCM_TAREFAGERCHK", $NRORG, 1, 10, true);
                                $params = array(
                                    "NRTAREFAGERCHK" => $nrtarefaagendchk,
                                    "NRGERENCIAMENTOCHK" => $nrgerenciamentochk,
                                    "NRTAREFACHK" => $atividade,
                                    "CDOPERADOR" => $row['FUNCIONARIO'],
                                    "DTHORATAREFAGERCHK" => $dataInicial->format('d/m/Y H:i'),
                                    "DSOBSTAREFAGERCHK" => null,
                                    "OPERADOR" => $this->Environment->getCdOperador()
                                );

                                $this->TarefasFactory->insertChkTarefa($params);

                                foreach ($param['NRCRITERIOTAREFACHK'] as $criterios) {
                                    //valida se o critério realmente pertence à atividade que está sendo inserida acima.
                                    $paramsValida = array(
                                        "NRTAREFACHK"      => $atividade,
                                        "NRCRITERIOTAREFA" => $criterios['NRCRITERIOTAREFA']
                                    );

                                    $valida = $this->TarefasFactory->validaCriterio($paramsValida);

                                    if ($valida > 0) {
                                        $nrcriteriostaragerchk = $this->newCode->getCodeByTableName("FCM_CRITERIOTARGERCHK", $NRORG, 1, 10, true);
                                        $params = array(
                                            "NRCRITERIOTARGERCHK" => $nrcriteriostaragerchk,
                                            "NMCRITERIOTARGERCHK" => $criterios['DSCRITERIOTAREFA'],
                                            "NRTAREFAGERCHK" => $nrtarefaagendchk,
                                            "NRCRITERIOTAREFACHK" => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                                            "OPERADOR" => $this->Environment->getCdOperador()
                                        );

                                        $this->TarefasFactory->insertChkCriterio($params);
                                    }
                                }
                            }
                        }
                }
            }


            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    public function inativarTarefas($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $param = implode(',', $parametros["NRS"]);
            $this->TarefasFactory->deletarTarefas($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getTarefasSelectAll()
    {
        return $this->TarefasFactory->getTarefasSelectAll();
    }

    public function getTarefasPopulaModal($param)
    {
        $param = array(
            "NRCHECKLISTAGEND" => $param['NRCHECKLISTAGEND']
        );
        return $this->TarefasFactory->getTarefasPopulaModal($param);
    }

    public function populaEscopo($param)
    {
        $param = implode(",", $param['NRTAREFACHK']);
        return $this->TarefasFactory->populaEscopo($param);
    }

    public function populaCriterios($param)
    {
        $param = implode(",", $param['NRTAREFACHK']);
        $retorno = $this->TarefasFactory->populaCriterios($param);
        $retorno2 = [];
        foreach ($retorno['data'] as $retorno) {
            array_push($retorno2, $retorno['NRCRITERIOTAREFACHK']);
        }

        return $retorno2;
    }

    // recebe no formato "dd/mm/aaaa 00:00"
    public function getDuracao($param)
    {
        // muda formado de "dd/mm/aaaa 00:00" para "dd-mm-aaaa 00:00" para possibilitar
        // uso da função strtotime
        $param['DTHORAAGEND'] = str_replace('/', '-', $param['DTHORAAGEND']);
        $param['DTHRFINALAGEND'] = str_replace('/', '-', $param['DTHRFINALAGEND']);

        // transforma as datas em string para segundos
        $param['DTHORAAGEND'] = strtotime($param['DTHORAAGEND']);
        $param['DTHRFINALAGEND'] = strtotime($param['DTHRFINALAGEND']);

        // descobre o intervalo de segundos entre uma data e outra
        $interval = $param['DTHORAAGEND'] - $param['DTHRFINALAGEND'];

        // caso seja um número negativo, transforma em absoluto (inteiro)
        $interval = abs($interval);

        // pega a quantidade de segundos e transforma no formato "00:00"
        $duration = sprintf('%02d:%02d', ($interval/ 3600), ($interval/ 60 % 60));

        return $duration;
    }

    public function getObsObrigatoria($param)
    {
        $param = array(
            "NRCHECKLISTAGEND" => $param['NRCHECKLISTAGEND']
        );
        return $this->TarefasFactory->getObsObrigatoria($param);
    }

    public function getCriarVistoria($param)
    {
        $param = array(
            "NRCHECKLISTAGEND" => $param['NRCHECKLISTAGEND']
        );
        return $this->TarefasFactory->getCriarVistoria($param);
    }
}
