<?php

namespace FM\Service;

use FM\Factory\Fm00030000TarefasRecorrentesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\Fm00004000TarefasService;

class Fm00030000TarefasRecorrentesService
{

    protected $Fm00030000TarefasRecorrentesFactory;
    protected $Environment;
    protected $newCode;
    protected $FuncionariosHierarqService;
    protected $Fm00004000TarefasService;
    protected $connection;

    public function __construct(EntityManager $entityManager, Fm00030000TarefasRecorrentesFactory $Fm00030000TarefasRecorrentesFactory, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode, Fm00004000TarefasService $Fm00004000TarefasService)
    {
        $this->connection                           = $entityManager->getConnection();
        $this->Fm00030000TarefasRecorrentesFactory  = $Fm00030000TarefasRecorrentesFactory;
        $this->FuncionariosHierarqService           = $FuncionariosHierarqService;
        $this->Environment                          = $Environment;
        $this->newCode                              = $newCode;
        $this->Fm00004000TarefasService             = $Fm00004000TarefasService;
    }


    public function insertTarefasRecorrentes($param)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $datasConflitantes = array();

            foreach ($param['NRCRITERIOTAREFACHK'] as $key => $criterio) {
                unset($param['NRCRITERIOTAREFACHK'][$key]['RNUM']);
                unset($param['NRCRITERIOTAREFACHK'][$key]['TOTAL_LINES']);
            }

            $dados = ($param['CONFLITO'] === true) ?  $this->montarDadosPosConflito($param) : $this->montarDadosTarefa($param);
            
            $parametros = $this->validaDiasUteis($dados);
            $paramValidado = $this->validarConflitos($parametros, true);
            

            if (count($paramValidado[1]) !== 0) {
                throw new \Exception("Operação bloqueada!<br><br> Existem conflitos de horário nas tarefas que está tentando inserir. Altere os horários, ou o Operador, ou desative a parametrização que permite inserir mais de uma tarefa para o mesmo horário.");
            }
            unset($paramValidado[1]);

            $paramValidado = $this->validarConflitos($paramValidado[0], false);

            if (count($paramValidado[1]) > 0) {
                array_push($datasConflitantes, $paramValidado[1]);
            }


            if ($param['RECORRENCIA'] !== 'SR') {
                $nrTarefaRecorrente = $this->newCode->getCodeByTableName("FCM_TAREFARECORRENTE", $NRORG, 1, 10, true);
                $paramValidado[0]['NRTAREFARECORRENTE'] = $nrTarefaRecorrente;

                $this->inserirTarefaRecorrente($paramValidado[0]);
                $this->Fm00004000TarefasService->insertNovaTarefa($paramValidado[0]);

                do {
                    if ($paramValidado[0]['TERMINARECO'] == 'N') {
                        $paramValidado = $this->calcularRecorrenciaPeriodo($paramValidado[0]);
                        $paramValidado = $this->validarConflitos($paramValidado, false);
                        if (count($paramValidado[1]) > 0) {
                            array_push($datasConflitantes, $paramValidado[1]);
                        }
                        $this->Fm00004000TarefasService->insertNovaTarefa($paramValidado[0]);
                    }
                } while ($paramValidado[0]['TERMINARECO'] == 'N');
            } else {
                $this->Fm00004000TarefasService->insertNovaTarefa($paramValidado[0]);
            }
            
            if (count($datasConflitantes) > 0) {
                $retorno = $this->formatarRetorno($paramValidado, $datasConflitantes);
                unset($datasConflitantes);
            } else {
                $retorno = 'OK';
            }

            $this->connection->commit();
            return $retorno;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function formatarRetorno($parametros, $datasConflitantes)
    {
        $backup = array();
        if ($parametros[0]['CONFLITO']) {
            unset($parametros[0]['DADOSTAREFAS']);
            $parametros[0]['DADOSTAREFAS'] = $datasConflitantes[0];
        } else {
            foreach ($datasConflitantes as $key => $mes) {
                foreach ($mes as $dia) {
                    $arr = [$dia['DATAINICIAL'], $dia['FUNCIONARIO'], $dia['NRLOCALCLIENTE']];
                    if (array_diff($arr, $backup)) {
                        array_push($parametros[0]['DADOSTAREFAS'], $dia);
                    }
                    $backup = $arr;
                }
            }
        }
        unset($parametros[1]);
        return $parametros[0];
    }

    public function inserirTarefaRecorrente($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $this->Fm00030000TarefasRecorrentesFactory->insertTarefasRecorrentes(array(
                "NRTAREFARECORRENTE"    => $parametros['NRTAREFARECORRENTE'],
                "NMTAREFA"              => $parametros['NOMETAREFA'],
                "NRCHECKLIST"           => $parametros['NRCHECKLIST'],
                "ITENSCHECKLIST"        => implode(",", $parametros['NRATIVIDADECHK']),
                "ESCOPOSTAREFA"         => implode(",", $parametros['NRESCOPOSTAREFA']),
                "ITENSTAREFA"           => implode(",", $parametros['NRTAREFACHK']),
                "CRITERIOS"             => json_encode($parametros['NRCRITERIOTAREFACHK']),
                "DIASSEMANA"            => $parametros['DIASEMANA'],
                "IDCRIARVISTORIA"       => $parametros['VISTORIA'],
                "NRORG"                 => $NRORG,
                "IDATIVO"               => "S",
                "CDOPERINCLUSAO"        => $operador,
                "NRORGINCLUSAO"         => $NRORG,
                "IDSITUACAO"            => "1",
                "CDCLIENTE"             => $parametros['CDCLIENTE'],
                "NRSEQCONTRATO"         => isset($parametros['DADOSTAREFAS'][0]['NRSEQCONTRATO']) ? $parametros['DADOSTAREFAS'][0]['NRSEQCONTRATO'] : null,
                "DADOSRECORRENCIA"      => json_encode($parametros['DADOSTAREFAS']),
                "IDTIPORECORRENCIA"     => $parametros['RECORRENCIA'],
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function calcularRecorrenciaPeriodo($parametros)
    {
        // Definir a frequência da tarefa
        switch ($parametros['RECORRENCIA']) {
            case 'Q':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'M':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'S':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'QP':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'MP':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'SP':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'P':
                $frequencia = new \DateInterval('P'.$parametros['QTDDIAS'].'D');
                break;
        }
        
        // Loop para adicionar as datas das próximas tarefas
        foreach ($parametros['DADOSTAREFAS'] as $key => $executor) {
            $dataInicio = new \DateTime($executor['DATAINICIALBKP']);
            $dataFim = new \DateTime($executor['DATAFINALBKP']);

            $dataInicio = $dataInicio->add($frequencia);
            $dataFim = $dataFim->add($frequencia);
            $executor['DATAINICIAL'] = $dataInicio->format('m/d/Y H:i');
            $executor['DATAFINAL'] = $dataFim->format('m/d/Y H:i');
            $parametros['DADOSTAREFAS'][$key] = $executor;
        }

        return $this->validaDiasUteis($parametros);
    }

    public function geraItervalo($dataInicio, $dataFim)
    {
        // Definir um intervalo de 1 dia
        $fim = new \DateTime($dataFim);
        $intervalo = new \DateInterval('P1D');
        $periodo = new \DatePeriod(new \DateTime($dataInicio), $intervalo, $fim->modify('+1 day'));

        $datas = array();
        foreach ($periodo as $data) {
            array_push($datas, $data->format('m/d/Y'));
        }
        return $datas;
    }

    public function validaDiasUteis($parametros)
    {
        foreach ($parametros['DADOSTAREFAS'] as $key => $exec) {
            $dataInicio = new \DateTime($exec['DATAINICIAL']);
            $dataFim = new \DateTime($exec['DATAFINAL']);

            $exec['DATAINICIALBKP'] = $dataInicio->format('m/d/Y H:i');
            $exec['DATAFINALBKP'] = $dataFim->format('m/d/Y H:i');

            $diaDaSemana = $dataInicio->format('N');
            $dt = $dataInicio->format('m/d/Y');

            // Verificar se a data é um dia útil
            if ($parametros['DIASEMANA'] == '2') {
                if ($diaDaSemana == 6) { // sábado
                    $dataInicio->add(new \DateInterval('P2D')); // soma dois dias para retornar à sexta-feira
                    $dataFim->add(new \DateInterval('P2D'));
                } elseif ($diaDaSemana == 7) { // domingo
                    $dataInicio->add(new \DateInterval('P1D')); // soma um dias para retornar ao último dia útil anterior, que é sexta-feira
                    $dataFim->add(new \DateInterval('P1D'));
                }
                $exec['GERATAREFA'] =  'S';
            } elseif ($parametros['DIASEMANA'] == '3') {
                $exec['GERATAREFA'] = ($diaDaSemana == 6 || $diaDaSemana == 7) ? 'N' : 'S';
                $parametros['DADOSTAREFAS'][$key] = $exec;
            } elseif ($parametros['DIASEMANA'] == '1') {
                $exec['GERATAREFA'] = 'S';
                $parametros['DADOSTAREFAS'][$key] = $exec;
            }

            $dataTermino = (isset($parametros['DTTERMINO'])) ? $parametros['DTTERMINO'] : $parametros['DATERANGE'][0];

            if (new \DateTime($dt) <= new \DateTime($dataTermino)) {
                $exec['DATAINICIAL'] = $dataInicio->format('m/d/Y H:i');
                $exec['DATAFINAL'] = $dataFim->format('m/d/Y H:i');
                $parametros['DADOSTAREFAS'][$key] = $exec;
                $parametros['TERMINARECO'] =  'N';
            } else {
                $parametros['TERMINARECO'] =  'S';
                unset($parametros['DADOSTAREFAS'][$key]);
            }
        }

        return $parametros;
    }

    public function validarConflitos($parametros, $front)
    {
        $datasConflitantes = array();
        if ($front) {
            for ($i = 0; $i < count($parametros['DADOSTAREFAS']); $i++) {
                for ($j = $i + 1; $j < count($parametros['DADOSTAREFAS']); $j++) {
                    if ($parametros['DADOSTAREFAS'][$i]['NRLOCALCLIENTE'] !== $parametros['DADOSTAREFAS'][$j]['NRLOCALCLIENTE'] &&
                        $parametros['DADOSTAREFAS'][$i]['FUNCIONARIO'] == $parametros['DADOSTAREFAS'][$j]['FUNCIONARIO']
                    ) {
                        if ($this->verificarConflitoDatas($parametros['DADOSTAREFAS'][$i], $parametros['DADOSTAREFAS'][$j])) {
                            $parametros['DADOSTAREFAS'][$i]['GERATAREFA'] = 'N';
                            $parametros['DADOSTAREFAS'][$j]['GERATAREFA'] = 'N';
                            $parametros['DADOSTAREFAS'][$i]['AGENDAMENTOATUAL'] = 'S';
                            $parametros['DADOSTAREFAS'][$j]['AGENDAMENTOATUAL'] = 'S';

                            array_push($datasConflitantes, $parametros['DADOSTAREFAS'][$i]);
                            array_push($datasConflitantes, $parametros['DADOSTAREFAS'][$j]);
                        }
                    }
                }
            }
        } else {
            $dadosBanco = $this->Fm00030000TarefasRecorrentesFactory->getAllTasks();
            foreach ($parametros['DADOSTAREFAS'] as $chave => $dadoFront) {
                foreach ($dadosBanco as $dadoBack) {
                    if ($dadoFront['FUNCIONARIO'] == $dadoBack['CDOPERADOR'] && $dadoFront['NRLOCALCLIENTE'] == $dadoBack['NRLOCALCLIENTE']) {
                        if ($this->verificarConflitoDatas($dadoFront, $dadoBack)) {
                            $dadoFront['GERATAREFA'] = 'N';
                            $dadoFront['AGENDAMENTOATUAL'] = 'N';
                            $parametros['DADOSTAREFAS'][$chave] = $dadoFront;
                            array_push($datasConflitantes, $dadoFront);
                        }
                    }
                }
            }
        }
        return [$parametros, $datasConflitantes];
    }

    public function verificarConflitoDatas($dadosX, $dadosY)
    {
        // Converter as strings de data/hora para objetos DateTime
        $inicio1 = new \DateTime($dadosX['DATAINICIAL']);
        $fim1 = new \DateTime($dadosX['DATAFINAL']);
        $inicio2 = new \DateTime($dadosY['DATAINICIAL']);
        $fim2 = new \DateTime($dadosY['DATAFINAL']);

        // Verificar se há conflito de datas
        if (($inicio1 >= $inicio2 && $inicio1 <= $fim2) ||
            ($fim1 >= $inicio2 && $fim1 <= $fim2) ||
            ($inicio1 <= $inicio2 && $fim1 >= $fim2)
        ) {
            return true;
        }
        return false;
    }

    public function montarDadosTarefa($parametros)
    {
        $intervaloDias = $this->geraItervalo($parametros['DATERANGE'][0], (isset($parametros['DATERANGE'][1]) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]));

        $parametros['DADOSTAREFAS'] = array();
        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            foreach ($intervaloDias as $dia) {
                $datas = $this->calcularDuracao($dia, $dados['HORAINICIO'], $dados['DURACAO']);
                $dados['DATAINICIAL'] = $datas[0];
                $dados['DATAFINAL'] = $datas[1];
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['DADOSTAREFAS'], $dados);
            }
        }
        return $parametros;
    }

    public function montarDadosPosConflito($parametros)
    {
        $parametros['DADOSTAREFAS'] = array();
        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            if ($dados['IGNORARTAREFA'] !== 'S') {
                $datas = $this->calcularDuracao($dados['DATAINICIAL'], $dados['HORAINICIO'], $dados['DURACAO']);
                $dados['DATAINICIAL'] = $datas[0];
                $dados['DATAFINAL'] = $datas[1];
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['DADOSTAREFAS'], $dados);
            } else {
                unset($parametros['EXECUTOR'][$key]);
            }
        }
        return $parametros;
    }

    public function calcularDuracao($dataInicial, $horaInicial, $duracao)
    {
        // converter a duração vinda do front para o formato esperado pelo método DateInterval
        $hora = intval(substr($duracao, 0, 2));
        $minuto = intval(substr($duracao, 2, 4));
        $totalMinutos = ($hora * 60) + $minuto;

        $horarioInicial = $dataInicial . " " . $horaInicial;

        $horarioInicial = new \DateTime($horarioInicial);
        $horarioFinal = clone ($horarioInicial);

        $horarioFinal->add(new \DateInterval('PT' . $totalMinutos . 'M'));

        return [$horarioInicial->format('m/d/Y H:i'), $horarioFinal->format('m/d/Y H:i')];
    }

    /* public function montaExecutorHora($dataInicial, $dataFinal, $params, $nrlocalcliente)
    {
        $NRORG =  $this->Environment->getNrOrg();
        $operador = $this->Environment->getCdOperador();
        $executorHora = array();
        $relatorioConflito = array();
        $recorrente = $this->Fm00030000TarefasRecorrentesFactory->getDuracao($params);
        $duracao = (intdiv($recorrente[0]['DURACAO'], 100) * 60)  + $recorrente[0]['DURACAO'] % 100;
        for ($inicio = $dataInicial; $inicio <= $dataFinal; $inicio = (date("m/d/Y", strtotime($inicio . ' +1 day')))) {
            $inicio = $inicio . " " . $recorrente[0]['HORARIO'];
            if ($this->validaConflitos($inicio, $recorrente) != true) {
                array_push($executorHora, array(
                    "DATAINICIAL" => date("d/m/Y H:i", strtotime($inicio)),
                    "DATAFINAL"   => date("d/m/Y H:i", strtotime($inicio . ' +' . $duracao . ' minute')),
                    "DURACAO"     => $duracao,
                    "FUNCIONARIO" => $recorrente[0]['CDOPERTAREFA'],
                    "ENCARREGADO" => $recorrente[0]['CDOPERVISTORIA'],
                    "NRLOCALCLIENTE" => $nrlocalcliente
                ));
            } else {
                $NRERROTAREFAREC = $this->newCode->getCodeByTableName("FCM_ERROSTAREFAREC", $NRORG, 1, 10, true);
                array_push($relatorioConflito, array(
                    "DTINICIAL"        => date("d/m/Y H:i", strtotime($inicio)),
                    "DTFINAL"          => date("d/m/Y H:i", strtotime($inicio . ' +' . $duracao . ' minute')),
                    "DURACAO"            => $duracao,
                    "CDOPERRESPONSAVEL"  => $recorrente[0]['CDOPERTAREFA'],
                    "CDOPERENCARREGADO"  => $recorrente[0]['CDOPERVISTORIA'],
                    "NMTAREFARECORRENTE" => $recorrente[0]['NMTAREFA'],
                    "NRTAREFARECORRENTE" => $recorrente[0]['NRTAREFARECORRENTE'],
                    "NRERROTAREFAREC"    => $NRERROTAREFAREC,
                    "NRORG" => $NRORG,
                    "IDATIVO" => 'S',
                    "NRORGINCLUSAO" => $NRORG,
                    "NRLOCALCLIENTE" => $nrlocalcliente,
                ));
            }
        }
        foreach ($relatorioConflito as $conflito) {
            $this->Fm00030000TarefasRecorrentesFactory->criaRelatorio($conflito);
        }

        return $executorHora;
    } */

    // public function calcularRecorrencia($tipoRecorrencia, $dataAtual, $diaSemana)
    // {
    //     if (!is_array($dataAtual)) {
    //         // Definir a data de início da tarefa
    //         $dataInicio = new \DateTime($dataAtual);

    //         // Definir a frequência da tarefa
    //         switch ($tipoRecorrencia) {
    //             case 'Q':
    //                 $frequencia = new \DateInterval('P14D');
    //                 break;
    //             case 'M':
    //                 $frequencia = new \DateInterval('P1M');
    //                 break;
    //             case 'S':
    //                 $frequencia = new \DateInterval('P179D');
    //                 break;
    //         }

    //         // Adicionar a frequência à data de início
    //         $data = $dataInicio->add($frequencia);

    //         $dataCriarTarefa[] = $data;
    //         $data = $data->format('m/d/Y');

    //         // Verificar se a data é um dia útil


    //         // Adicionar a data das tarefa ao array
    //         return array(
    //             "DTINICIOPROXGERACAO"   => $data,
    //             "DTFINALPROXGERACAO"    => null,
    //             "DATACRIACAOTAREFA"     => $this->validaDiasUteis($dataCriarTarefa, $diaSemana),
    //         );
    //     } else {
    //         $retorno = $this->calcularRecorrenciaPeriodo($tipoRecorrencia, $dataAtual, $diaSemana);
    //     }
    // }

    // public function getTarefasRecorrentes()
    // {
    //     $retorno = $this->Fm00030000TarefasRecorrentesFactory->getDadosTarefaRecorrente();
    //     $this->validaErroRecorrente($retorno);
    //     return $retorno;
    // }

    // public function validaErroRecorrente()
    // {
    //     $this->connection->beginTransaction();
    //     try {
    //         $retorno =  $this->Fm00030000TarefasRecorrentesFactory->getErroRelatorio();
    //         foreach ($retorno as $validador) {
    //             if ($validador['VALIDA'] != null) {
    //                 $iderro = 'S';
    //                 $this->Fm00030000TarefasRecorrentesFactory->setIdErro($iderro, $validador['NRTAREFARECORRENTE']);
    //             } else {
    //                 $iderro = 'N';
    //                 $this->Fm00030000TarefasRecorrentesFactory->setIdErro($iderro, $validador['NRTAREFARECORRENTE']);
    //             }
    //         }
    //         $this->connection->commit();
    //     } catch (\Exception $e) {
    //         $this->connection->rollback();
    //         throw $e;
    //     }
    // }

    // public function getRelatoriosConflitos($filtro)
    // {
    //     $filter = "";
    //     if ($filtro['NRTAREFARECORRENTE'] != 'S') {
    //         $filter = "WHERE NRTAREFARECORRENTE = " . $filtro['NRTAREFARECORRENTE'];
    //     }
    //     return $this->Fm00030000TarefasRecorrentesFactory->getRelatoriosConflitos($filter);
    // }

    // public function delRegistroRelatorio($parametro)
    // {
    //     $param = implode(',', $parametro["NRS"]);
    //     $this->Fm00030000TarefasRecorrentesFactory->delRegistroRelatorio($param);
    // }

    // public function mudarStatusTarefaRecorrente($param)
    // {

    //     $this->connection->beginTransaction();
    //     try {
    //         $paramDB = array(
    //             "NRTAREFARECORRENTE" => $param['NRTAREFARECORRENTE'],
    //             "IDSITUACAO" => null,
    //         );
    //         if ($param['TIPO'] == "PAUSAR") {
    //             $paramDB['IDSITUACAO'] = 2;
    //         } elseif ($param['TIPO'] == "INICIAR") {
    //             $paramDB['IDSITUACAO'] = 1;
    //         } elseif ($param['TIPO'] == "CANCELAR") {
    //             $paramDB['IDSITUACAO'] = 3;
    //         }

    //         $this->Fm00030000TarefasRecorrentesFactory->mudarStatusTarefaRecorrente($paramDB);

    //         switch ($paramDB['IDSITUACAO']) {
    //             case 1:
    //                 break;
    //             case 3:
    //                 $param["DATAFINAL"] = "";
    //                 // no break
    //             case 2:
    //                 $this->deletarPeriodoTarefaRecorrente($param);
    //                 if ($param["DATAFINAL"] != null) {
    //                     $this->dataRetorno($param);
    //                 }
    //                 break;
    //         }

    //         $this->connection->commit();
    //         $response = "OK";
    //     } catch (\Exception $e) {
    //         $this->connection->rollback();
    //         $response = "Erro";
    //     }
    //     return $response;
    // }

    // public function dataRetorno($param)
    // {
    //     $dataReinicio = date("d/m/Y", strtotime($param["DATAFINAL"] . '+ 1 days'));
    //     $NR = $param["NRTAREFARECORRENTE"];

    //     $this->Fm00030000TarefasRecorrentesFactory->dataRetorno($dataReinicio, $NR);
    // }

    // public function inativarTarefasRecorrentes($parametros)
    // {
    //     $this->connection->beginTransaction();
    //     try {
    //         $params = implode(',', $parametros["NRS"]);
    //         $filter = "";
    //         $filter .= "AND DTHORATAREFAAGEND >= TO_DATE('" . $parametros['DATAINICIO'] . "' ,'DD/MM/YYYY')";

    //         $this->Fm00030000TarefasRecorrentesFactory->deletarPeriodoTarefaRecorrente($params, $filter);

    //         $parametro = array(
    //             "NRTAREFARECORRENTE" => $params,
    //             "IDSITUACAO"         => '3',
    //         );
    //         $this->Fm00030000TarefasRecorrentesFactory->mudarStatusTarefaRecorrente($parametro);

    //         $this->connection->commit();
    //     } catch (\Exception $e) {
    //         $this->connection->rollback();
    //         throw $e;
    //     }
    // }

    // public function editarTarefaRecorrente($param)
    // {
    //     $this->connection->beginTransaction();
    //     try {
    //         $params = array(
    //             "NRTAREFARECORRENTE"    => $param['NRTAREFARECORRENTE'],
    //             "DIASSEMANA"            => implode(",", $param['DIASSEMANA']),
    //             "HORARIO"               => $param['HORARIO'],
    //             "DURACAO"               => $param['DURACAO'],
    //             "CDOPERTAREFA"          => $param['CDOPERTAREFA'],
    //             "CDOPERVISTORIA"        => $param['CDOPERVISTORIA'],
    //             "CDOPERULTATU"          => $this->Environment->getCdOperador()
    //         );

    //         $this->Fm00030000TarefasRecorrentesFactory->editarTarefaRecorrente($params);
    //         $this->connection->commit();
    //     } catch (\Exception $e) {
    //         $this->connection->rollback();
    //         throw $e;
    //     }
    // }

    // public function deletarPeriodoTarefaRecorrente($param)
    // {

    //     $dataFinal = date("d/m/Y", strtotime($param['DATAFINAL']));
    //     $filtrar = "";
    //     if (!$param['DATAFINAL'] || $dataFinal == null) {
    //         $filtrar .= "AND DTHORATAREFAAGEND >= TO_DATE('" . $param['DATAINICIAL'] . "' ,'DD/MM/YYYY')";
    //     } else {
    //         $filtrar .= "AND DTHORATAREFAAGEND BETWEEN TO_DATE('" . $param['DATAINICIAL'] . "' ,'DD/MM/YYYY') AND TO_DATE('" . $dataFinal . "','DD/MM/YYYY')";
    //     }

    //     $this->Fm00030000TarefasRecorrentesFactory->deletarPeriodoTarefaRecorrente($param["NRTAREFARECORRENTE"], $filtrar);
    // }

    // public function recriarTarefasRecorrentes($params)
    // {
    //     $NRORG = $this->Environment->getNrOrg();
    //     $operador = $this->Environment->getCdOperador();
    //     $dadosTarefasRecorrentes = $this->Fm00030000TarefasRecorrentesFactory->pegarTodosDados(strval($params['NRTAREFARECORRENTE']));
    //     $dataInicial = (date("m/d/Y", strtotime($params['DATAINICIAL'])) < date("m/d/Y") ? date("m/d/Y") : $params['DATAINICIAL']);

    //     if (date("m/d/Y", strtotime($dataInicial)) < date("m/25/Y")) {
    //         $valida =   $this->Fm00030000TarefasRecorrentesFactory->validaPeriodo(strval($params['NRTAREFARECORRENTE']), date("d/m/Y", strtotime($dataInicial)));
    //         $NRSEQCONTRATO = [array("NRSEQCONTRATO" => intval($dadosTarefasRecorrentes[0]['NRSEQCONTRATO']))];
    //         $DADOSTAREFAS = $this->montaDADOSTAREFAS($dataInicial, date("m/t/Y"), $params['NRTAREFARECORRENTE'], $dadosTarefasRecorrentes[0]['LOCALTAREFA']);

    //         $criaTarefa =  array(
    //             "NRSEQCONTRATO"       => $NRSEQCONTRATO[0],
    //             "DADOSTAREFAS"        => $DADOSTAREFAS,
    //             "NRTAREFARECORRENTE"  => strval($dadosTarefasRecorrentes[0]['NRTAREFARECORRENTE']),
    //             "NOMETAREFA"          => $dadosTarefasRecorrentes[0]['NMTAREFA'],
    //             "NRTIPOLOCALCLI"      => explode(",", $dadosTarefasRecorrentes[0]['TPLOCALTAREFA']),
    //             "NRLOCALCLIENTE"      => [array($dadosTarefasRecorrentes[0]['LOCALTAREFA'])[0]],
    //             "NRCHECKLIST"         => $dadosTarefasRecorrentes[0]['NRCHECKLIST'],
    //             "NRATIVIDADECHK"      => explode(",", $dadosTarefasRecorrentes[0]['ITENSCHECKLIST']),
    //             "NRESCOPOSTAREFA"     => explode(",", $dadosTarefasRecorrentes[0]['ESCOPOSTAREFA']),
    //             "NRTAREFACHK"         => explode(",", $dadosTarefasRecorrentes[0]['ITENSTAREFA']),
    //             "NRCRITERIOTAREFACHK" => json_decode($dadosTarefasRecorrentes[0]['CRITERIOS'], true),
    //             "DIASEMANA"           => explode(",", $dadosTarefasRecorrentes[0]['DIASSEMANA']),
    //             "IDCRIARVISTORIA"     => $dadosTarefasRecorrentes[0]['IDCRIARVISTORIA'],
    //             "NRORG"               => $NRORG,
    //             "IDATIVO"             => "S",
    //             "CDOPERINCLUSAO"      => $operador,
    //             "NRORGINCLUSAO"       => $NRORG,
    //             "IDSITUACAO"          => "1",
    //             "DATERANGE"           => [date("m/d/Y"), date("m/t/Y")],
    //             "OBSERVACOES"         => $dadosTarefasRecorrentes[0]['IDOBRIGAOBSERVACAO'],
    //             "VISTORIA"            => ($dadosTarefasRecorrentes[0]['IDCRIARVISTORIA'] == 'S') ? '1' : '2',
    //             "CDCLIENTE"           => $dadosTarefasRecorrentes[0]['CDCLIENTE'],

    //         );

    //         $this->Fm00004000TarefasService->insertNovaTarefa($criaTarefa);
    //     } elseif (date("m/d/Y", strtotime($dataInicial)) >= date("m/25/Y") && date("m/d/Y", strtotime($dataInicial)) < date("m/t/Y", strtotime(date("m/1/Y") . " +1month"))) {
    //         $dataFinal = date("m/t/Y", strtotime(date("m/1/Y") . " +1month"));
    //         $NRSEQCONTRATO = [array("NRSEQCONTRATO" => intval($dadosTarefasRecorrentes[0]['NRSEQCONTRATO']))];
    //         $DADOSTAREFAS = $this->montaDADOSTAREFAS($dataInicial, $dataFinal, $params['NRTAREFARECORRENTE'], $dadosTarefasRecorrentes[0]['LOCALTAREFA']);

    //         $criaTarefa =  array(
    //             "NRSEQCONTRATO"       => $NRSEQCONTRATO[0],
    //             "DADOSTAREFAS"        => $DADOSTAREFAS,
    //             "NRTAREFARECORRENTE"  => strval($dadosTarefasRecorrentes[0]['NRTAREFARECORRENTE']),
    //             "NOMETAREFA"          => $dadosTarefasRecorrentes[0]['NMTAREFA'],
    //             "NRTIPOLOCALCLI"      => explode(",", $dadosTarefasRecorrentes[0]['TPLOCALTAREFA']),
    //             "NRLOCALCLIENTE"      => [array($dadosTarefasRecorrentes[0]['LOCALTAREFA'])[0]],
    //             "NRCHECKLIST"         => $dadosTarefasRecorrentes[0]['NRCHECKLIST'],
    //             "NRATIVIDADECHK"      => explode(",", $dadosTarefasRecorrentes[0]['ITENSCHECKLIST']),
    //             "NRESCOPOSTAREFA"     => explode(",", $dadosTarefasRecorrentes[0]['ESCOPOSTAREFA']),
    //             "NRTAREFACHK"         => explode(",", $dadosTarefasRecorrentes[0]['ITENSTAREFA']),
    //             "NRCRITERIOTAREFACHK" => json_decode($dadosTarefasRecorrentes[0]['CRITERIOS'], true),
    //             "DIASEMANA"           => explode(",", $dadosTarefasRecorrentes[0]['DIASSEMANA']),
    //             "IDCRIARVISTORIA"     => $dadosTarefasRecorrentes[0]['IDCRIARVISTORIA'],
    //             "NRORG"               => $NRORG,
    //             "IDATIVO"             => "S",
    //             "CDOPERINCLUSAO"      => $operador,
    //             "NRORGINCLUSAO"       => $NRORG,
    //             "IDSITUACAO"          => "1",
    //             "DATERANGE"           => [date("m/d/Y"), date("m/t/Y")],
    //             "OBSERVACOES"         => $dadosTarefasRecorrentes[0]['IDOBRIGAOBSERVACAO'],
    //             "VISTORIA"            => ($dadosTarefasRecorrentes[0]['IDCRIARVISTORIA'] == 'S') ? '1' : '2',
    //             "CDCLIENTE"           => $dadosTarefasRecorrentes[0]['CDCLIENTE'],

    //         );
    //         $this->Fm00004000TarefasService->insertNovaTarefa($criaTarefa);
    //     }

    //     $this->mudarStatusTarefaRecorrente(array(
    //         "TIPO"               => "INICIAR",
    //         "NRTAREFARECORRENTE" => $params["NRTAREFARECORRENTE"],
    //     ));

    //     $retiraReinicio = array(
    //         "NRTAREFARECORRENTE" =>  $params["NRTAREFARECORRENTE"],
    //         "DTHORAREINICIA" => "",
    //     );
    //     $this->Fm00030000TarefasRecorrentesFactory->retirarReinicio($retiraReinicio);
    // }

    // public function criaRecorrencia()
    // {
    //     $NRS = $this->Fm00030000TarefasRecorrentesFactory->getTarefasRecorrente();
    //     $data = array(
    //         "DATAINICIAL" => date("m/1/Y", strtotime(date("m/1/Y") . " +1month")),
    //     );
    //     foreach ($NRS as $nr) {
    //         $this->recriarTarefasRecorrentes(array_merge($nr, $data));
    //     }
    // }

    // public function validaConflitos($dataInicio, $recorrente)
    // {
    //     $tarefas = $this->Fm00030000TarefasRecorrentesFactory->getAllTasks();
    //     $duracao = (intdiv($recorrente[0]['DURACAO'], 100) * 60) + $recorrente[0]['DURACAO'] % 100;
    //     foreach ($tarefas as $tarefa) {
    //         $dig = strtotime(date("m/d/Y H:i", strtotime($tarefa['DTHORAAGEND'])));
    //         $dfg = strtotime(date("m/d/Y H:i", strtotime($tarefa['DTHRFINALAGEND'])));
    //         $din = strtotime(date("m/d/Y H:i", strtotime($dataInicio)));
    //         $dfn = strtotime(date("m/d/Y H:i", strtotime($dataInicio . ' +' . $duracao . 'minute')));

    //         if ($recorrente[0]['CDOPERTAREFA'] == $tarefa['CDOPERADOR']) {
    //             if (($din >= $dig && $din <= $dfg || $dfn >= $dig && $dfn <= $dfg) ||
    //                 $dig >= $din && $dig <= $dfn || $dig >= $din && $dig <= $dfn
    //             ) {
    //                 return true;
    //             }
    //         }
    //     }
    // }
}
