import { Http, IEventParam, Metadata } from '@zeedhi/core';
import {
    DialogService, LoadingService, TextInput,
    } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class Fm00002TipoLocalController {
    public validaTipoLocal({ row }: IEventParam<TekGrid>) {
        if (row.VALIDA != null) {
            return false;
        }
        return true;
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRTIPOLOCALCLI: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMTIPOLOCALCLI_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public atualizarTipoLocal({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/atualizarTipoLocal', {
                DRG: component.getEditedRows(),
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public afterModify() {
        Metadata.getInstance<TekGrid>('tek-grid-tipo-local').datasource.get();
    }

    public excluirTipoLocal({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00002TipoLocalController.aposConfirmarDelete', 'title');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-tipo-local');

        LoadingService.show();
        Http.post('/excluirTipoLocal', {
            NRS: component.selectedRows.map((element: any) => element.NRTIPOLOCALCLI),
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public validacao({ row }: any) {
        return row.VALIDA !== null;
    }
}
